import { Component, OnInit, NgZone } from "@angular/core";
import { Http, Headers, RequestOptions } from "@angular/http";
import { Observable } from "rxjs/Observable";
import { ImageFormat } from "ui/enums";
import * as Camera from "camera";
import * as Toast from "nativescript-toast";
import "rxjs/Rx";
var FileSystem = require("file-system");
var BackgroundHttp = require("nativescript-background-http");
var MD5 = require("blueimp-md5");

@Component({
    selector: "ns-app",
    templateUrl: "app.component.html",
})
export class AppComponent implements OnInit {

    public images: Array<string>;

    public constructor(private http: Http, private zone: NgZone) {
        this.images = [];
    }

    public ngOnInit() {
        this.list()
            .subscribe(result => {
                this.images = result;
            });
    }

    public takePicture() {
        Camera.takePicture({saveToGallery: false, width: 320, height: 240}).then(picture => {
            let folder = FileSystem.knownFolders.documents();
            let path = FileSystem.path.join(folder.path, MD5(new Date()) + ".png");
            picture.saveToFile(path, ImageFormat.png, 60);
            this.upload("http://localhost:3000/upload", "upload", path)
                .subscribe(result => {
                    this.zone.run(() => {
                        this.images.push(path.replace(/^.*[\\\/]/, ''));
                    });
                }, error => {
                    console.dump(error);
                });
        });
    }

    public upload(destination: string, filevar: string, filepath: string) {
        return new Observable((observer: any) => {
            let session = BackgroundHttp.session("file-upload");
            let request = {
                url: destination,
                method: "POST"
            };
            let params = [{ "name": filevar, "filename": filepath, "mimeType": "image/png" }];
            let task = session.multipartUpload(params, request);
            task.on("complete", (event) => {
                let file = FileSystem.File.fromPath(filepath);
                file.remove().then(result => {
                    observer.next("Uploaded `" + filepath + "`");
                    observer.complete();
                }, error => {
                    observer.error("Could not delete `" + filepath + "`");
                });
            });
            task.on("error", event => {
                console.dump(event);
                observer.error("Could not upload `" + filepath + "`.  " + event.eventName);
            });
        });
    }

    public list(): Observable<any> {
        return this.http.get("http://localhost:3000/list")
            .map(result => result.json())
            .map(result => result.filter(s => s.name.substr(s.name.length - 4) == ".png"))
            .map(result => result.map(s => s.name));
    }

    public remove(index: number) {
        Toast.makeText("Removing image...").show();
        let headers = new Headers({ "Content-Type": "application/json" });
        let options = new RequestOptions({ headers: headers });
        this.http.delete("http://localhost:3000/delete?filename=" + this.images[index])
            .map(result => result.json())
            .subscribe(result => {
                this.images.splice(index, 1);
            });
    }

}
